function [all_results] = analyze_wav(filename, samplesPerFft, instrument, deb)

%[all_results]=ANALYZE_WAV(PL) plots the waveform and spectrogram, and
%creates a structure all_results with the useful information.
%
% PropertyList PL;
%   filename- the name of the file to be analyzed may use with or without
%       'wav' extension, but all files are treated as wav files.  The file
%       to be analyzed should be in the same folder as this function.
%   samplesPerFft- the number of samples each fft contains.
%   instrument- for instrument specific data.  Possible choices now are
%       violin and piano.
%   deb- integer.  Will be set to 0 if not specified.  If deb is specified
%       (and >0), two figures will open.  The first shows the raw audio 
%       signal, the second is the spectrogram. 
%==========================================================================

%Debugging check.  Makes sure that there is 4 input arguments, and appends
% wav extension if necessary.  Also sets up the figure placement.
if nargin < 4
    deb = 0;
end

[path,name,ext] = fileparts(filename);
if isempty(ext)
    filename = [filename '.wav'];
end

fig = get_next_fig;


%sets instrument-specific parameters
if isequal(instrument,'violin')
   lowFreq = 196;
   highFreq = 2093;
elseif isequal(instrument, 'piano')
   lowFreq = 27.5;
   highFreq = 3951.07;
end

%reading in the wav file.
[samples,samplesPerSec,bitsPerSample] = wavread(filename);

%creates a vector with the time each sample was taken.
sampleTimes= [1:length(samples)]./samplesPerSec;


%Display raw data (audio signal).
if deb
    ready_plot(get_next_fig,2,sprintf('%s',filename));
    plot(sampleTimes,samples,'b-');
    xlabel('Time [sec]');
    ylabel('Amplitude');
    scale_xaxis(0,ceil(last(sampleTimes)),fig,fig+1);
end

overlap = floor(samplesPerFft*.8);

%Calls the built in function specgram to create the info needed
%   for drawing a spectrogram.
[fftSlices, fftBinFrequencies,times] = specgram(samples, ...        %data to analyze
                                                samplesPerFft, ...     %local FFTs resolution
                                                samplesPerSec, ...  %not used in acutal calculation
                                                samplesPerFft, ...
                                                overlap);            %samples per FFT 
 
% Creates a variable nBins that is used in drawing the spectrogram.                                            
nBins= length(fftSlices);
                                            
 if deb
    %Draw specgram (stole this from specgram.m code).
 %   ready_plot(get_next_fig,2,sprintf('%s Spectral Analysis', filename));
 %   select = (nBins - 1)' * samplesPerSec / samplesPerFft;
 %   imagesc(times, fftBinFrequencies, 20*log10(abs(fftSlices) + eps));
 %   axis xy; 
 %   colormap(jet);
 %   xlabel('Time [sec]');
 %   ylabel('Frequency');
 %   scale_xaxis(ceil(last(sampleTimes)),fig,fig+1,0);
end

%Structure with useful data.
all_results = struct('samples',samples, 'sampleTimes_seconds', sampleTimes,... 
                      'samplingInterval_seconds', samplesPerFft/samplesPerSec,...
                     'samplingFreq_Hz', samplesPerSec, 'frequencyRes_Hz', samplesPerSec/samplesPerFft,...
                     'minCyclesPerInterval', lowFreq*samplesPerFft/samplesPerSec,...
                     'lowestFreq', lowFreq, 'highestFreq', highFreq, ...
                     'fftSliceTimes_seconds',times',...
                     'fftSlices',fftSlices,'fftBins_hz',fftBinFrequencies,...
                     'dataFileName', filename);