function [all_results,clipped_results] = analyze_wav_for_violin(filename, samplesPerFft, deb)
%[all_results,clipped_results] = analyze_wav_for_violin(filename,samplesPerFft, deb*(0)): returns specgrams for wav file 
%===========================================================================================================================
%filename: .wav extension appended if not already there, must be in same directory as this function's .m file
%
%all_results and clipped_results: structures that contain specgrams and related info (fields are self-documenting names)
%
%if deb > 0, opens two figures, each w/zoom buttons (Zx and Zy for zooming only on x and y axis respectively) cruising 
%around in the data:
%  - first is audio signal (time vs. amplitude read straight out of wav file)
%  - second is specgram (time vs. frequency; color reflects how much power in given frequency, darker means more power)
%
%note: difference between all and clipped is just that clipped contains only those FFT bins that lie w/in 
%  the violin's fundamental frequency range whereas all contains every frequency returned by matlab's 
%  specgram function

if nargin < 3
    deb = 0;
end

[path,name,ext] = fileparts(filename);
if isempty(ext)
    filename = [filename '.wav'];
end

fig = get_next_fig;

%Read in audio WAV data
disp(sprintf('Reading in %s....',filename));
a = cputime;
[samples,samplesPerSec,bitsPerSample] = wavread(['OldWavFiles' filesep filename]);
b = cputime;
nSamples = length(samples);
secsPerSample = 1/samplesPerSec;
sampleTimes = [1:1:nSamples]*secsPerSample;
%I bet data doesn't need to be real in the function I'm writing here, 
%but it certainly would be if its WAV data.
assert(isreal(samples),'Data not real!'); 
disp(sprintf('Time to read %d samples of %d-bit data, sampled at %g [Hz], was %g [sec].',...
     nSamples,bitsPerSample,samplesPerSec,b-a));

%Display raw data.
if deb
    ready_plot(get_next_fig,2,sprintf('%s',filename));
    plot(sampleTimes,samples,'b-');
    xlabel('Time [sec]');
    ylabel('Amplitude');
    scale_xaxis(0,ceil(last(sampleTimes)),fig,fig+1);
end


%Unique to the violin:             
highCyclesPerSec = 2093; %4 octaves above middle C
lowCyclesPerSec = 196;  %Low G string 


%Size of our local FFTs and how much data they're based on:
%if(secsPerLowCycle > secsPerFft)
   % assert(0,'You cannot get the resolution you want!');
    %else
   %you know what, my stuff is crap.
   %disp('ignore all the stuff above, its not helpful w/fft and violin transcription');
   %disp(sprintf('lindsays suggestion: use nSamples %d',2200));
   % samplesPerFft = secsPerFft * samplesPerSec;
   % I CHANGED THIS TO 4400 TO GET BETTER RESOLUTION -KLP
   samplesPerFft = 1470;
   %end

%The purpose of the following display is to provide user with a notion of how well
%the short-term FFTs we're going to calculate capture what's going on wrt the
%instrument, the digitized data, etc.
%disp(sprintf('With a sampling frequency of %g [samples/sec] we get %g [sec/sample].',...
 %             samplesPerSec, secsPerSample));
%disp(sprintf('With a tempo of %g [beats/min],', beatsPerMin));
%disp(sprintf('     a duration resolution of %g [notes/beat],', beatsPerMin, notesPerBeat));
%disp(sprintf('     an inter-tone resolution of %g [FFTs/note],', fftsPerNote));
%disp(sprintf('     and a steady-state resolution of %d [periods/FFT],', nPeriods));
%disp(sprintf('     we need to calculate %g [FFTs/sec], which gives us %g [secs/FFT]. ',...
%              fftsPerSec, 1 / fftsPerSec));
%disp(sprintf('     or %g [samples/FFT].', samplesPerSec / fftsPerSec));
%disp(sprintf('Since violin notes range from %g to %g Hz (%g to %g [sec])',...
%              lowCyclesPerSec, highCyclesPerSec, 1 / lowCyclesPerSec, 1 / highCyclesPerSec));
%disp(sprintf('      the sampling frequency exceeds the highest note by a factor of %g.',...
%              samplesPerSec / highCyclesPerSec));
%disp(sprintf('Furthermore, estimating an FFT with %d periods of the lowest notes data',...
%              nPeriods));
%disp(sprintf('      will take %g [secs/FFT], which is %g [samples/FFT]',...
%              secsPerLowCycle, samplesPerFft));

%Do spectral analysis. (E84 Team: This is what is doing the work so much 
%faster than your for loop code.)
nBinsInFft = floor(samplesPerFft);
window = floor(samplesPerFft);
%Here specgram's overlap is the deafult (window/2);
[fftSlices, fftBinFrequencies,times] = specgram(samples, ...        %data to analyze
                                                nBinsInFft, ...     %local FFTs resolution
                                                samplesPerSec, ...  %not used in acutal calculation
                                                window);            %samples per FFT 
[nBins, nFfts] = size(fftSlices);

%It might be very useful in the future to consider only the results in the region
%of fundamental fftBinFrequencies that the violin can play, so I provide this data also.
clippedInds = find(lowCyclesPerSec <= fftBinFrequencies & fftBinFrequencies <= highCyclesPerSec);
clippedFftSlices = fftSlices(clippedInds,:);
clippedFftBinFrequences = fftBinFrequencies(clippedInds);
nClippedBins = size(clippedFftBinFrequences);

if deb
    %Draw specgram (stole this from specgram.m code).
    ready_plot(get_next_fig,2,sprintf('%s Spectral Analysis', filename));
    select = (nBins - 1)' * samplesPerSec / nBinsInFft;
    imagesc(times, fftBinFrequencies, 20*log10(abs(fftSlices) + eps));
    axis xy; 
    colormap(jet);
    xlabel('Time [sec]');
    ylabel('Frequency');
    scale_xaxis(ceil(last(sampleTimes)),fig,fig+1,0);
end
  
%Return useful data.
all_results = struct('samples',samples,'sampleTimes_seconds',([1:1:nSamples]')*secsPerSample,...
                     'fftSlices',fftSlices,'fftBins_hz',fftBinFrequencies,...
                     'fftSliceTimes_seconds',times',...
                     'secondsPerFftSlice',window*secsPerSample,'samplesPerFftSlice',window,...
                     'highestViolinTone_Hz',highCyclesPerSec,'lowestViolinTone_Hz',lowCyclesPerSec,...
                     'sampling_Hz',samplesPerSec,'samplingResolution',bitsPerSample,...
                     'dataFilename',filename);
clipped_results = struct('samples',samples,'sampleTimes_seconds',([1:1:nSamples]')*secsPerSample,...
                         'fftSlices',clippedFftSlices,'fftBins_hz',clippedFftBinFrequences,...
                         'fftSliceTimes_seconds',times',...
                         'secondsPerFftSlice',window*secsPerSample,'samplesPerFftSlice',window,...
                         'highestViolinTone_Hz',highCyclesPerSec,'lowestViolinTone_Hz',lowCyclesPerSec,...
                         'sampling_Hz',samplesPerSec,'samplingResolution',bitsPerSample,...
                         'dataFilename',filename);
