function [classIds,fftSliceTimes] = song_to_templates(songName,nBins,absThreshold,diffThreshold)
%song_to_templates(songName,nBins,absThreshold,diffThreshold): classifies song, shows its raw data, and some distance stats
%============================================================================================================================
%nBins:        constrols resolution w/which one can chop distance to template into bins
%absThreshold: threshold BELOW WHICH distance to nearest template not considered acceptable for classification.
%              this value is specified with respect to which bin is to be used for the cdf cutoff
%              note: zero means ignore this consideration when classifying and nBins means threshold everything
%diffThreshold:threshold BELOW WHICH distance between two nearest templates not considered acceptable for classification
%              this value is also specified with respect to which bin is to be used for the cdf cutoff
%              note: zero means ignore this consideration when classifying and nBins means threshold everything
%----------------------------------------------------------------------------------------------------------------------------
%the way this function classifies notes is very dependent upon what thresholds are used!

[templateBins,templateFfts,templateNames] = get_note_templates;
nTemplates = length(templateNames);

assert(ismember(absThreshold,0:1:nBins),'bogus abs threshold bin value');
assert(ismember(diffThreshold,0:1:nBins),'bogus diff threshold bin value');

nFft = 1470;
deb = 0;
[fftSlices,fftSliceTimes,samples,sampleTimes] = convert_song_to_slices(songName,nFft,deb);
nData = num_rows(fftSlices);

%for each new spectrum to classify, build distance vector from it to each template
dists = zeros(nData,nTemplates);
for i = 1:1:nData
    data = fftSlices{i};
    thisDists = zeros(1,nTemplates);
    for j = 1:1:nTemplates
        thisDists(1,j) = spectral_distance(templateFfts{j},data);
    end
    dists(i,:) = thisDists;
end

sortedDists = zeros(nData,nTemplates);
for i = 1:1:nData
    sortedDists(i,:) = sort(dists(i,:));
end

[abs_pdf,abs_cdf,abs_x] = pdf_cdf_estimate(sortedDists(:,1),nBins);
if isequal(absThreshold,0)
    %threshold everything
    a_thresh = 0;
    a_cdf = 0;
elseif isequal(absThreshold,nBins)
    %add on some padding to end of last bin so we never trigger on this
    a_thresh = last(abs_x) + abs_x(2) - abs_x(1);
    a_cdf = 1;
else
    %pick threshold as midpoint between two adjacent bins
    abs_x_append = [0 abs_x];
    a_thresh = mean([abs_x_append(absThreshold),abs_x_append(absThreshold+1)]);
    abs_cdf_append = [0 abs_cdf];
    a_cdf = mean([abs_cdf_append(absThreshold),abs_cdf_append(absThreshold+1)]);
end

firstDiff = sortedDists(:,2)-sortedDists(:,1);
[diff_pdf,diff_cdf,diff_x] = pdf_cdf_estimate(firstDiff,nBins);
if isequal(diffThreshold,0)
    %threshold on nothing
    d_thresh = 0;
    d_cdf = 0;
elseif isequal(diffThreshold,nBins)
    %add on some padding to end of last bin so we always trigger on this
    d_thresh = last(diff_x) + diff_x(2) - diff_x(1);
    d_cdf = 1;
else
    diff_x_append = [0 diff_x];
    d_thresh = mean([diff_x_append(diffThreshold),diff_x_append(diffThreshold+1)]);
    diff_cdf_append = [0 diff_cdf];
    d_cdf = mean([diff_cdf_append(diffThreshold),diff_cdf_append(diffThreshold+1)]);
end

%determine ith order statistic in cdf for absThreshold and diffThreshold
classIds = zeros(nData,1);
for i = 1:1:nData
    thisDists = dists(i,:);
    classIds(i) = classify_with_template(thisDists,templateNames,a_thresh,d_thresh);
end

nr = 4; nc = 1;
ready_subplot(get_next_fig,nr,nc);

subplot(nr,nc,1);
title(sprintf('song %s',songName));
doProbs = 1;
simple_hist(0,sortedDists(:,1),nBins,doProbs);
xlabel('closest template distance (for abs)');
ylabel('pdf');

subplot(nr,nc,2);
plot(abs_x,abs_cdf,'b.-');
xlabel('closest template distance (for abs)');
ylabel('cdf');

subplot(nr,nc,3);
simple_hist(0,firstDiff,nBins,doProbs);
xlabel('distance between 2 closest templates (diff)');
ylabel('pdf');

subplot(nr,nc,4);
plot(diff_x,diff_cdf,'b.-');
xlabel('distance between 2 closest templates (diff)');
ylabel('cdf');

nr = 3; nc = 1;
ready_subplot(get_next_fig,nr,nc);

subplot(nr,nc,1);
title(sprintf('song %s absCdf %.3g (bin %d) diffCdf %.3g (bin %d)',songName,a_cdf,absThreshold,d_cdf,diffThreshold));
plot(fftSliceTimes,classIds,'k-');
scale_subplots(1,nr,nc,struct('yMin',-5,'yMax',max(classIds)+2));
xlabel('fft data slice times');
ylabel('classification');

subplot(nr,nc,2);
plot(fftSliceTimes,sortedDists(:,1),'k-');
plot(fftSliceTimes,sortedDists(:,2),'b--');
xlabel('fft data slice times');
ylabel('top two distances');

subplot(nr,nc,3);
plot(sampleTimes,samples,'k-');
xlabel('raw data sample times');
ylabel('amplitude');

scale_subplots(1:1:nr*nc,nr,nc,struct('xMin',0,'xMax',max(fftSliceTimes)));

return;


